@@ -20,7 +20,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgacqctl.c,v 1.78 2003/04/09 20:44:17 tom Exp tom $
+ *  $Id: epgacqctl.c,v 1.85 2004/07/11 19:02:59 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGCTL
@@ -64,18 +64,22 @@ typedef struct
    EPGACQ_MODE    userMode;
    EPGACQ_PASSIVE passiveReason;
    EPGACQ_PHASE   cyclePhase;
+   EPGACQ_PHASE   stopPhase;
    uint           cycleIdx;
    uint           cniCount;
    uint           cniTab[MAX_MERGED_DB_COUNT];
    time_t         dumpTime;
    time_t         chanChangeTime;
    uint           inputSource;
+   uint           currentSlicerType;
+   bool           autoSlicerType;
    bool           haveWarnedInpSrc;
 } EPGACQCTL_STATE;
 
 static EPGACQCTL_STATE  acqCtl = {ACQSTATE_OFF, ACQMODE_FOLLOW_UI, ACQMODE_FOLLOW_UI};
 static EPGDB_STATS      acqStats;
 static bool             acqStatsUpdate;
+static bool             acqVpsPdcUpdate[VPSPDC_REQ_COUNT];
 static EPGACQ_DESCR     acqNetDescr;
 static bool             acqContextIsPeek;
 static EPGDB_QUEUE      acqDbQueue;
@@ -113,6 +117,7 @@ static void EpgAcqCtl_StatisticsReset( void )
    memset(&acqStats, 0, sizeof(acqStats));
 
    memset(&acqNetDescr, 0, sizeof(acqNetDescr));
+   memset(&acqVpsPdcUpdate, FALSE, sizeof(acqVpsPdcUpdate));
 
    acqStats.acqStartTime  = time(NULL);
    acqStats.histIdx       = STATS_HIST_WIDTH - 1;
@@ -195,24 +200,20 @@ static void EpgAcqCtl_StatisticsUpdate( void )
       else
          memset(pHist, 0, sizeof(*pHist));
 
-      // maintain history of netwop coverage variance for cycle state machine
-      if (ACQMODE_IS_CYCLIC(acqCtl.mode))
+      for (stream=0; stream <= 1; stream++)
       {
-         for (stream=0; stream <= 1; stream++)
-         {
-            if (acqStats.varianceHist[stream].count < VARIANCE_HIST_COUNT)
-            {  // history buffer not yet filled -> just append the new sample
-               idx = acqStats.varianceHist[stream].count;
-               acqStats.varianceHist[stream].count += 1;
-            }
-            else
-            {  // history buffer filled -> insert into ring buffer after the last written sample
-               acqStats.varianceHist[stream].lastIdx = (acqStats.varianceHist[stream].lastIdx + 1) % VARIANCE_HIST_COUNT;
-               idx = acqStats.varianceHist[stream].lastIdx;
-            }
-
-            acqStats.varianceHist[stream].buf[idx] = acqStats.count[stream].variance;
+         if (acqStats.varianceHist[stream].count < VARIANCE_HIST_COUNT)
+         {  // history buffer not yet filled -> just append the new sample
+            idx = acqStats.varianceHist[stream].count;
+            acqStats.varianceHist[stream].count += 1;
          }
+         else
+         {  // history buffer filled -> insert into ring buffer after the last written sample
+            acqStats.varianceHist[stream].lastIdx = (acqStats.varianceHist[stream].lastIdx + 1) % VARIANCE_HIST_COUNT;
+            idx = acqStats.varianceHist[stream].lastIdx;
+         }
+
+         acqStats.varianceHist[stream].buf[idx] = acqStats.count[stream].variance;
       }
    }
 }
@@ -257,6 +258,9 @@ const EPGDB_STATS * EpgAcqCtl_GetAcqStats( void )
       {  // retrieve additional data from TTX packet decoder
          TtxDecode_GetStatistics(&acqStats.ttx.ttxPkgCount,
                                  &acqStats.ttx.epgPkgCount, &acqStats.ttx.epgPagCount);
+         EpgStreamGetStatistics(&acqStats.stream);
+
+         acqStats.lastStatsUpdate = time(NULL);
       }
 
       pAcqStats = &acqStats;
@@ -275,8 +279,8 @@ const EPGDB_STATS * EpgAcqCtl_GetAcqStats( void )
 //
 const EPGDB_ACQ_VPS_PDC * EpgAcqCtl_GetVpsPdc( VPSPDC_REQ_ID clientId )
 {
-   static bool updateVect[VPSPDC_REQ_COUNT] = {FALSE};
    EPGDB_ACQ_VPS_PDC * result = NULL;
+   uint idx;
 
    if (clientId < VPSPDC_REQ_COUNT)
    {
@@ -285,13 +289,16 @@ const EPGDB_ACQ_VPS_PDC * EpgAcqCtl_GetVpsPdc( VPSPDC_REQ_ID clientId )
          // poll for new VPS/PDC data
          if (EpgAcqCtl_ProcessVps())
          {
-            memset(updateVect, FALSE, sizeof(updateVect));
+            for (idx = 0; idx < VPSPDC_REQ_COUNT; idx++)
+            {
+               acqVpsPdcUpdate[idx] = TRUE;
+            }
          }
 
-         // if there are reults which have not been given yet to the client, return them
-         if ( (updateVect[clientId] == FALSE) || (clientId == VPSPDC_REQ_DAEMON) )
+         // if there are results which have not been given yet to the client, return them
+         if ( acqVpsPdcUpdate[clientId] || (clientId == VPSPDC_REQ_DAEMON) )
          {
-            updateVect[clientId] = TRUE;
+            acqVpsPdcUpdate[clientId] = FALSE;
             result = &acqStats.vpsPdc;
          }
       }
@@ -313,12 +320,17 @@ const EPGDB_ACQ_VPS_PDC * EpgAcqCtl_GetVpsPdc( VPSPDC_REQ_ID clientId )
 //
 static void EpgAcqCtl_NetStatsUpdate( void )
 {
+   uint idx;
    bool aiFollows;
 
    if (EpgAcqClient_GetNetStats(&acqStats, &acqNetDescr, &aiFollows))
    {
       ifdebug2(acqNetDescr.dbCni != EpgDbContextGetCni(pAcqDbContext), "EpgAcqCtl-NetStatsUpdate: stats dbCni=0x%04X != acq db CNI=0x%04X", acqNetDescr.dbCni, EpgDbContextGetCni(pAcqDbContext));
 
+      for (idx = 0; idx < VPSPDC_REQ_COUNT; idx++)
+      {
+         acqVpsPdcUpdate[idx] = TRUE;
+      }
       if (aiFollows == FALSE)
       {  // no AI transmitted (no reception, or AI unchanged) -> trigger status update from here
          UiControlMsg_AcqEvent(ACQ_EVENT_STATS_UPDATE);
@@ -331,8 +343,14 @@ static void EpgAcqCtl_NetStatsUpdate( void )
 //
 void EpgAcqCtl_AddNetVpsPdc( const EPGDB_ACQ_VPS_PDC * pVpsPdcUpd )
 {
+   uint idx;
+
    acqStats.vpsPdc = *pVpsPdcUpd;
 
+   for (idx = 0; idx < VPSPDC_REQ_COUNT; idx++)
+   {
+      acqVpsPdcUpdate[idx] = TRUE;
+   }
    UiControlMsg_AcqEvent(ACQ_EVENT_VPS_PDC);
 
    dprintf5("EpgAcqCtl-VpsPdcCallback: %02d.%02d. %02d:%02d (0x%04X)\n", (acqStats.vpsPdc.pil >> 15) & 0x1F, (acqStats.vpsPdc.pil >> 11) & 0x0F, (acqStats.vpsPdc.pil >>  6) & 0x1F, (acqStats.vpsPdc.pil      ) & 0x3F, acqStats.vpsPdc.cni);
@@ -347,7 +365,7 @@ void EpgAcqCtl_UpdateNetProvList( uint cniCount, const uint * pCniList )
    if ( (acqCtl.userMode == ACQMODE_FOLLOW_UI) ||
         (acqCtl.userMode == ACQMODE_FOLLOW_MERGED) )
    {
-      EpgAcqCtl_SelectMode(ACQMODE_FOLLOW_UI, cniCount, pCniList);
+      EpgAcqCtl_SelectMode(ACQMODE_FOLLOW_UI, ACQMODE_PHASE_COUNT, cniCount, pCniList);
    }
 }
 #endif  // USE_DAEMON
@@ -748,6 +766,16 @@ static bool EpgAcqCtl_TuneProvider( uint freq, uint cni )
    // remember time of channel change
    acqCtl.chanChangeTime = time(NULL);
 
+#ifdef USE_PROXY
+   // XXX FIXME
+   BtDriver_SetChannelProfile( VBI_CHANNEL_PRIO_BACKGROUND,
+                               ((acqCtl.cyclePhase == ACQMODE_PHASE_MONITOR) ? VBI_CHN_SUBPRIO_INITIAL : VBI_CHN_SUBPRIO_CHECK),
+                               ((acqCtl.cyclePhase == ACQMODE_PHASE_NOWNEXT) ? 3*60 : 25*60),
+                               ((acqCtl.cyclePhase == ACQMODE_PHASE_MONITOR) ? 60 : 10) );
+#else
+   BtDriver_SetChannelProfile(VBI_CHANNEL_PRIO_BACKGROUND, 0, 0, 0);
+#endif
+
    // tune onto the provider's channel (before starting acq, to avoid catching false data)
    // always set the input source - may have been changed externally (since we don't hog the video device permanently)
    if ( BtDriver_TuneChannel(acqCtl.inputSource, freq, FALSE, &isTuner) )
@@ -785,6 +813,12 @@ static bool EpgAcqCtl_TuneProvider( uint freq, uint cni )
          acqCtl.mode = ACQMODE_FORCED_PASSIVE;
          acqCtl.passiveReason = ACQPASSIVE_NO_TUNER;
       }
+
+      if (acqCtl.autoSlicerType)
+      {  // in auto-mode fall back to default slicer for every channel change
+         acqCtl.currentSlicerType = VBI_SLICER_TRIVIAL;
+         BtDriver_SelectSlicer(acqCtl.currentSlicerType);
+      }
    }
    else
    {
@@ -981,6 +1015,12 @@ static void EpgAcqCtl_InitCycle( void )
          break;
    }
 
+   // limit to max phase (because acq. will stop upon advance from this phase)
+   if (acqCtl.cyclePhase > acqCtl.stopPhase)
+   {
+      acqCtl.cyclePhase = acqCtl.stopPhase;
+   }
+
    // use the first provider in the list
    acqCtl.cycleIdx = 0;
 }
@@ -1045,8 +1085,10 @@ static void EpgAcqCtl_AdvanceCyclePhase( bool forceAdvance )
         (wrongCni == FALSE) &&
         (EpgScan_IsActive() == FALSE) &&
         ( ACQMODE_IS_CYCLIC(acqCtl.mode) ||
+          (acqCtl.stopPhase != ACQMODE_PHASE_COUNT) ||
           ((acqCtl.mode == ACQMODE_FORCED_PASSIVE) && ACQMODE_IS_CYCLIC(acqCtl.userMode) && (acqCtl.passiveReason == ACQPASSIVE_ACCESS_DEVICE)) ) &&
-        (acqCtl.cniCount > 1) )
+        ( (acqCtl.cniCount > 1) ||
+          (acqCtl.stopPhase != ACQMODE_PHASE_COUNT) ) )
    {
       // determine if acq for the current phase is complete
       // note: all criteria must have a decent timeout to catch abnormal cases
@@ -1069,8 +1111,8 @@ static void EpgAcqCtl_AdvanceCyclePhase( bool forceAdvance )
          case ACQMODE_PHASE_STREAM2:
          case ACQMODE_PHASE_MONITOR:
             quote   = (((acqStats.count[0].ai + acqStats.count[1].ai) > 0) ?
-                       (((double)acqStats.count[0].sinceAcq + (double)acqStats.count[1].sinceAcq) /
-                        (acqStats.count[0].ai + acqStats.count[1].ai)) : 100.0);
+                       (((double)acqStats.count[0].sinceAcq + acqStats.count[1].sinceAcq) /
+                        ((double)acqStats.count[0].ai + acqStats.count[1].ai)) : 100.0);
             advance = (quote >= MIN_CYCLE_QUOTE) &&
                       (acqStats.count[0].variance < MIN_CYCLE_VARIANCE) &&
                       (acqStats.count[1].variance < MIN_CYCLE_VARIANCE) &&
@@ -1093,10 +1135,14 @@ static void EpgAcqCtl_AdvanceCyclePhase( bool forceAdvance )
             acqCtl.cycleIdx += 1;
             dprintf2("EpgAcqCtl: advance to CNI #%d of %d\n", acqCtl.cycleIdx, acqCtl.cniCount);
          }
+         else if (acqCtl.cyclePhase >= acqCtl.stopPhase)
+         {  // max. phase reached -> stop acq.
+            EpgAcqCtl_Stop();
+         }
          else
          {  // phase complete for all databases -> switch to next
             // determine next phase
-            switch (acqCtl.mode)
+            switch (acqCtl.userMode)
             {
                case ACQMODE_CYCLIC_012:
                   if (acqCtl.cyclePhase < ACQMODE_PHASE_MONITOR)
@@ -1127,9 +1173,11 @@ static void EpgAcqCtl_AdvanceCyclePhase( bool forceAdvance )
             acqCtl.cycleIdx = 0;
          }
 
-         // note about parameter "TRUE": open the new db immediately because
-         // it may need to be accessed, e.g. to reset rep counters
-         EpgAcqCtl_UpdateProvider(TRUE);
+         if (acqCtl.state != ACQSTATE_OFF)
+         { // note about parameter "TRUE": open the new db immediately because
+           // it may need to be accessed, e.g. to reset rep counters
+            EpgAcqCtl_UpdateProvider(TRUE);
+         }
       }
    }
    assert((acqCtl.userMode != ACQMODE_FORCED_PASSIVE) && (acqCtl.userMode < ACQMODE_COUNT));
@@ -1165,7 +1213,8 @@ bool EpgAcqCtl_CheckDeviceAccess( void )
 // ---------------------------------------------------------------------------
 // Select acquisition mode and provider list
 //
-bool EpgAcqCtl_SelectMode( EPGACQ_MODE newAcqMode, uint cniCount, const uint * pCniTab )
+bool EpgAcqCtl_SelectMode( EPGACQ_MODE newAcqMode, EPGACQ_PHASE maxPhase,
+                           uint cniCount, const uint * pCniTab )
 {
    bool restart;
    bool result = FALSE;
@@ -1187,6 +1236,7 @@ bool EpgAcqCtl_SelectMode( EPGACQ_MODE newAcqMode, uint cniCount, const uint * p
          // check if the same parameters are already set -> if yes skip
          // note: compare actual mode instead of user mode, to reset if acq is stalled
          if ( (newAcqMode != acqCtl.mode) ||
+              (maxPhase   != acqCtl.stopPhase) ||
               (cniCount   != acqCtl.cniCount) ||
               (memcmp(acqCtl.cniTab, pCniTab, cniCount * sizeof(*pCniTab)) != 0) )
          {
@@ -1203,6 +1253,7 @@ bool EpgAcqCtl_SelectMode( EPGACQ_MODE newAcqMode, uint cniCount, const uint * p
             acqCtl.cniCount         = cniCount;
             acqCtl.userMode         = newAcqMode;
             acqCtl.mode             = newAcqMode;
+            acqCtl.stopPhase        = maxPhase;
             memcpy(acqCtl.cniTab, pCniTab, sizeof(acqCtl.cniTab));
 
             // reset acquisition and start with the new parameters
@@ -1248,24 +1299,42 @@ bool EpgAcqCtl_SelectMode( EPGACQ_MODE newAcqMode, uint cniCount, const uint * p
 }
 
 // ---------------------------------------------------------------------------
-// Select input source: tuner or composite
-// - this info is very important to the acq control, since only when a
-//   tuner is the input source, the channel can be controlled; else only
-//   passive acq mode is possible
-// - not used in passive and network acq modes
+// TV card config: select input source and slicer type
+// - distinguishing tuner/external is important to acq control, because
+//   + when a tuner is input source, the channel can be controlled
+//   + else only passive acq mode is possible
+//   note: parameter is not used in passive and network acq modes
+// - slicer type is passed through to driver, except for "automatic"
 //
-bool EpgAcqCtl_SetInputSource( uint inputIdx )
+bool EpgAcqCtl_SetInputSource( uint inputIdx, uint slicerType )
 {
    bool result;
 
    acqCtl.inputSource = inputIdx;
 
+   // save slicer type; if automatic start with "trivial"
+   if (slicerType == VBI_SLICER_AUTO)
+   {
+      acqCtl.autoSlicerType = TRUE;
+      acqCtl.currentSlicerType = VBI_SLICER_TRIVIAL;
+   }
+   else
+   {
+      acqCtl.autoSlicerType = FALSE;
+      acqCtl.currentSlicerType = slicerType;
+   }
+
    if ( (acqCtl.state != ACQSTATE_OFF) &&
         (acqCtl.mode != ACQMODE_PASSIVE) &&
         (acqCtl.mode != ACQMODE_NETWORK) )
    {
       acqCtl.haveWarnedInpSrc = FALSE;
 
+      if (acqCtl.autoSlicerType == FALSE)
+      {
+         BtDriver_SelectSlicer(acqCtl.currentSlicerType);
+      }
+
       EpgAcqCtl_InitCycle();
       result = EpgAcqCtl_UpdateProvider(TRUE);
 
@@ -1309,7 +1378,12 @@ void EpgAcqCtl_DescribeAcqState( EPGACQ_DESCR * pAcqState )
          if (acqCtl.state != ACQSTATE_RUNNING)
          {
             if (now - lastAi > ACQ_DESCR_STALLED_TIMEOUT)
-               pAcqState->state = ACQDESCR_NO_RECEPTION;
+            {
+               if (acqCtl.state >= ACQSTATE_WAIT_AI)
+                  pAcqState->state = ACQDESCR_DEC_ERRORS;
+               else
+                  pAcqState->state = ACQDESCR_NO_RECEPTION;
+            }
             else
                pAcqState->state = ACQDESCR_STARTING;
          }
@@ -1391,12 +1465,14 @@ static bool EpgAcqCtl_AiCallback( const AI_BLOCK *pNewAi )
          // store the tuner frequency if none is known yet
          if ( (pAcqDbContext->tunerFreq == 0) && (acqCtl.mode != ACQMODE_NETWORK) )
          {
+            uint  freq, chan;
+            bool  isTuner;
             // try to obtain the frequency from the driver (not supported by all drivers)
-            pAcqDbContext->tunerFreq = BtDriver_QueryChannel();
-            if (pAcqDbContext->tunerFreq != 0)
+            if ( BtDriver_QueryChannel(&freq, &chan, &isTuner) && (isTuner) && (freq != 0) )
             {  // store the provider channel frequency in the rc/ini file
-               debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)(pAcqDbContext->tunerFreq & 0xffffff)/16, ai_cni);
-               UiControlMsg_NewProvFreq(ai_cni, pAcqDbContext->tunerFreq);
+               debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)(freq & 0xffffff)/16, ai_cni);
+               pAcqDbContext->tunerFreq = freq;
+               UiControlMsg_NewProvFreq(ai_cni, freq);
             }
          }
 
@@ -1508,12 +1584,14 @@ static bool EpgAcqCtl_AiCallback( const AI_BLOCK *pNewAi )
               (acqCtl.state == ACQSTATE_RUNNING) &&
               (acqCtl.mode != ACQMODE_NETWORK) )
          {
+            uint  freq, chan;
+            bool  isTuner;
             // try to obtain the frequency from the driver (not supported by all drivers)
-            pAcqDbContext->tunerFreq = BtDriver_QueryChannel();
-            if (pAcqDbContext->tunerFreq != 0)
+            if ( BtDriver_QueryChannel(&freq, &chan, &isTuner) && (isTuner) && (freq != 0) )
             {  // store the provider channel frequency in the rc/ini file
-               debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)(pAcqDbContext->tunerFreq & 0xffffff)/16, ai_cni);
-               UiControlMsg_NewProvFreq(ai_cni, pAcqDbContext->tunerFreq);
+               debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)(freq & 0xffffff)/16, ai_cni);
+               pAcqDbContext->tunerFreq = freq;
+               UiControlMsg_NewProvFreq(ai_cni, freq);
             }
          }
       }
@@ -1768,12 +1846,17 @@ bool EpgAcqCtl_ProcessPackets( void )
 {
    uint pageNo;
    bool stopped;
+   time_t now;
    bool result = FALSE;
 
    if (pAcqDbContext != NULL)
    {
       if (acqCtl.mode != ACQMODE_NETWORK)
       {
+         // query if VBI device has been freed by higher-prio users
+         if (BtDriver_QueryChannelToken())
+            EpgAcqCtl_CheckDeviceAccess();
+
          // check if new data is available in the VBI ringbuffer
          if (TtxDecode_CheckForPackets(&stopped))
          {  // assemble VBI lines, i.e. TTX packets to EPG blocks and put them in the queue
@@ -1783,6 +1866,21 @@ bool EpgAcqCtl_ProcessPackets( void )
                EpgAcqCtl_ChannelChange(TRUE);
                EpgAcqCtl_StatisticsUpdate();
             }
+
+            // check if the current slicer type is adequate
+            if ( (acqCtl.autoSlicerType) &&
+                 (acqCtl.currentSlicerType + 1 < VBI_SLICER_COUNT) )
+            {
+               now = time(NULL);
+               if ( (now >= acqCtl.chanChangeTime +
+                     (acqCtl.currentSlicerType - VBI_SLICER_TRIVIAL + 1) * EPGACQCTL_SLICER_INTV) &&
+                    (EpgStreamCheckSlicerQuality() == FALSE) )
+               {
+                  debug4("EpgAcqCtl: upgrading slicer type to #%d after %d secs (provider #%d of %d)", acqCtl.currentSlicerType + 1, (int)(now - acqCtl.chanChangeTime), acqCtl.cycleIdx, acqCtl.cniCount);
+                  acqCtl.currentSlicerType += 1;
+                  BtDriver_SelectSlicer(acqCtl.currentSlicerType);
+               }
+            }
             result = TRUE;
          }
 
