@@ -24,7 +24,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgdbif.c,v 1.40 2002/02/13 21:00:20 tom Exp tom $
+ *  $Id: epgdbif.c,v 1.46 2003/09/19 22:07:55 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGDB
@@ -153,6 +153,20 @@ void EpgDbSetAiUpdateTime( const EPGDB_CONTEXT * dbc, time_t acqTimestamp )
       fatal0("EpgDb-SetAiUpdateTime: illegal NULL ptr param");
 }
 
+// ---------------------------------------------------------------------------
+// Install a callback for notifications about incoming PI blocks
+// - NULL may be passed as function pointer to remove a previous callback
+//
+void EpgDbSetPiAcqCallback( EPGDB_CONTEXT * dbc, EPGDB_PI_ACQ_CB * pCb )
+{
+   if (dbc != NULL)
+   {
+      dbc->pPiAcqCb = pCb;
+   }
+   else
+      fatal0("EpgDb-SetPiAcqCallback: illegal NULL ptr param");
+}
+
 // ---------------------------------------------------------------------------
 // Get a pointer to the AI block in the database
 //
@@ -416,6 +430,121 @@ const PI_BLOCK * EpgDbSearchPi( CPDBC dbc, time_t start_time, uchar netwop_no )
    return NULL;
 }
 
+// ---------------------------------------------------------------------------
+// Search for a PI block in the database with a given minimum start time
+// - if the filter context parameter is not NULL, only blocks that match
+//   these settings are considered
+//
+const PI_BLOCK * EpgDbSearchFirstPiAfter( CPDBC dbc, time_t min_time, EPGDB_TIME_SEARCH_MODE startOrStop,
+                                          const FILTER_CONTEXT *fc )
+{
+   EPGDB_BLOCK * pBlock;
+
+   if ( EpgDbIsLocked(dbc) )
+   {
+      pBlock = dbc->pFirstPi;
+
+      if (startOrStop == STARTING_AT)
+      {
+         while ( (pBlock != NULL) && (pBlock->blk.pi.start_time < min_time))
+         {
+            pBlock = pBlock->pNextBlock;
+         }
+
+         if (fc != NULL)
+         {  // skip forward until a matching block is found
+            while ( (pBlock != NULL) &&
+                    (EpgDbFilterMatches(dbc, fc, &pBlock->blk.pi) == FALSE) )
+            {
+               pBlock = pBlock->pNextBlock;
+            }
+         }
+      }
+      else if (startOrStop == RUNNING_AT)
+      {
+         while ((pBlock != NULL) && (pBlock->blk.pi.stop_time <= min_time))
+         {
+            pBlock = pBlock->pNextBlock;
+         }
+
+         if (fc != NULL)
+         {  // skip forward until a matching block is found
+            // note: must keep on checking stop time (unlike start time search PI may follow on
+            //       different networks which have an earlier stop time due to shorter running time)
+            while ( (pBlock != NULL) &&
+                    ( (pBlock->blk.pi.stop_time <= min_time) ||
+                      (EpgDbFilterMatches(dbc, fc, &pBlock->blk.pi) == FALSE) ))
+            {
+               pBlock = pBlock->pNextBlock;
+            }
+         }
+      }
+      else
+         fatal1("EpgDb-SearchFirstPiAfter: invalid time search mode %d", startOrStop);
+
+
+      if (pBlock != NULL)
+      {
+         return &pBlock->blk.pi;
+      }
+   }
+   else
+      fatal0("EpgDb-SearchFirstPiAfter: DB not locked");
+
+   return NULL;
+}
+
+// ---------------------------------------------------------------------------
+// Search for a PI block in the database with a given maximum start time
+// - if the filter context parameter is not NULL, only blocks that match
+//   these settings are considered
+//
+const PI_BLOCK * EpgDbSearchFirstPiBefore( CPDBC dbc, time_t min_time, EPGDB_TIME_SEARCH_MODE startOrStop,
+                                           const FILTER_CONTEXT *fc )
+{
+   EPGDB_BLOCK * pBlock;
+
+   if ( EpgDbIsLocked(dbc) )
+   {
+      pBlock = dbc->pLastPi;
+
+      if (startOrStop == STARTING_AT)
+      {
+         while ((pBlock != NULL) && (pBlock->blk.pi.start_time >= min_time))
+         {
+            pBlock = pBlock->pPrevBlock;
+         }
+      }
+      else if (startOrStop == RUNNING_AT)
+      {
+         while ((pBlock != NULL) && (pBlock->blk.pi.stop_time > min_time))
+         {
+            pBlock = pBlock->pPrevBlock;
+         }
+      }
+      else
+         fatal1("EpgDb-SearchFirstPiBefore: invalid time search mode %d", startOrStop);
+
+      if (fc != NULL)
+      {  // skip forward until a matching block is found
+         while ( (pBlock != NULL) &&
+                 (EpgDbFilterMatches(dbc, fc, &pBlock->blk.pi) == FALSE) )
+         {
+            pBlock = pBlock->pPrevBlock;
+         }
+      }
+
+      if (pBlock != NULL)
+      {
+         return &pBlock->blk.pi;
+      }
+   }
+   else
+      fatal0("EpgDb-SearchFirstPiBefore: DB not locked");
+
+   return NULL;
+}
+
 // ---------------------------------------------------------------------------
 // Search for the first matching PI block in database
 // - The first is the the block with the lowest start time.
@@ -552,6 +681,75 @@ const PI_BLOCK * EpgDbSearchPrevPi( CPDBC dbc, const FILTER_CONTEXT *fc, const P
       return NULL;
 }
 
+// ---------------------------------------------------------------------------
+// Count matching PI blocks starting with a given block
+// - note: the returned count includes the given block (if it matches)
+//
+uint EpgDbCountPi( CPDBC dbc, const FILTER_CONTEXT *fc, const PI_BLOCK * pPiBlock )
+{
+   const EPGDB_BLOCK * pBlock = NULL;
+   uint  count = 0;
+
+   if ( EpgDbIsLocked(dbc) )
+   {
+      if (pPiBlock != NULL)
+      {
+         pBlock = (const EPGDB_BLOCK *)((ulong)pPiBlock - BLK_UNION_OFF);
+
+         while (pBlock != NULL)
+         {
+            if ( (fc == NULL) ||
+                 EpgDbFilterMatches(dbc, fc, &pBlock->blk.pi) )
+            {
+               count += 1;
+            }
+            pBlock = pBlock->pNextBlock;
+         }
+      }
+      else
+         fatal0("EpgDb-CountPi: illegal NULL ptr param");
+   }
+   else
+      fatal0("EpgDb-CountPi: DB not locked");
+
+   return count;
+}
+
+// ---------------------------------------------------------------------------
+// Count matching PI blocks before a given block
+// - note: the returned count does not include the given block
+//
+uint EpgDbCountPrevPi( CPDBC dbc, const FILTER_CONTEXT *fc, const PI_BLOCK * pPiBlock )
+{
+   const EPGDB_BLOCK * pBlock = NULL;
+   uint  count = 0;
+
+   if ( EpgDbIsLocked(dbc) )
+   {
+      if (pPiBlock != NULL)
+      {
+         pBlock = (const EPGDB_BLOCK *)((ulong)pPiBlock - BLK_UNION_OFF);
+         pBlock = pBlock->pPrevBlock;
+
+         while (pBlock != NULL)
+         {
+            if ( (fc == NULL) ||
+                 EpgDbFilterMatches(dbc, fc, &pBlock->blk.pi) )
+            {
+               count += 1;
+            }
+            pBlock = pBlock->pPrevBlock;
+         }
+      }
+      else
+         fatal0("EpgDb-CountPrevPi: illegal NULL ptr param");
+   }
+   else
+      fatal0("EpgDb-CountPrevPi: DB not locked");
+
+   return count;
+}
+
 // ---------------------------------------------------------------------------
 // Search for the PI block with the given PIL
 // - also search through the obsolete blocks, because the planned program
@@ -582,6 +780,33 @@ const PI_BLOCK * EpgDbSearchPiByPil( CPDBC dbc, uchar netwop_no, uint pil )
    return NULL;
 }
 
+// ----------------------------------------------------------------------------
+// Decode VPS/PDC timestamp from Nextview EPG format into standard struct
+// - note: the VPS label represents local time (not UTC!)
+// - returns TRUE is the label did contain a valid time
+//
+bool EpgDbGetVpsTimestamp( struct tm * pVpsTime, uint pil, time_t startTime )
+{
+   bool result;
+
+   memcpy(pVpsTime, localtime(&startTime), sizeof(*pVpsTime));
+
+   pVpsTime->tm_sec  = 0;
+   pVpsTime->tm_min  =  (pil      ) & 0x3F;
+   pVpsTime->tm_hour =  (pil >>  6) & 0x1F;
+   pVpsTime->tm_mon  = ((pil >> 11) & 0x0F) - 1; // range 0-11
+   pVpsTime->tm_mday =  (pil >> 15) & 0x1F;
+   // the rest of the elements (year, day-of-week etc.) stay the same as in
+   // start_time; since a VPS label usually has the same date as the actual
+   // start time this should work out well.
+
+   result = ( (pVpsTime->tm_min < 60) && (pVpsTime->tm_hour < 24) &&
+              (pVpsTime->tm_mon >= 0) && (pVpsTime->tm_mon < 12) &&
+              (pVpsTime->tm_mday >= 1) && (pVpsTime->tm_mday <= 31) );
+
+   return result;
+}
+
 // ---------------------------------------------------------------------------
 // Get the index of a PI in his netwop's chain
 // - The index of a netwop's first PI is 0, or 1, if it's start time lies in
@@ -670,12 +895,36 @@ uint EpgDbGetProgIdx( CPDBC dbc, const PI_BLOCK * pPiBlock )
 uchar EpgDbGetStream( const void * pUnion )
 {
    const EPGDB_BLOCK *pBlock;
-   uchar stream;
 
-   pBlock = (const EPGDB_BLOCK *)((ulong)pUnion - BLK_UNION_OFF);
-   stream = pBlock->stream;
+   if (pUnion != NULL)
+   {
+      pBlock = (const EPGDB_BLOCK *)((ulong)pUnion - BLK_UNION_OFF);
 
-   return stream;
+      return pBlock->stream;
+   }
+   else
+      fatal0("EpgDb-GetStream: illegal NULL ptr param");
+
+   return 0;
+}
+
+// ---------------------------------------------------------------------------
+// Returns the timestamp of last update for a PI block
+//
+time_t EpgDbGetPiUpdateTime( const PI_BLOCK * pPiBlock )
+{
+   const EPGDB_BLOCK *pBlock;
+
+   if (pPiBlock != NULL)
+   {
+      pBlock = (const EPGDB_BLOCK *)((ulong)pPiBlock - BLK_UNION_OFF);
+
+      return pBlock->updTimestamp;
+   }
+   else
+      fatal0("EpgDb-GetPiUpdateTime: illegal NULL ptr param");
+
+   return (time_t) 0;
 }
 
 // ---------------------------------------------------------------------------
@@ -766,7 +1015,7 @@ bool EpgDbGetStat( CPDBC dbc, EPGDB_BLOCK_COUNT * pCount, time_t * acqMinTime, u
    const AI_NETWOP *pNetwops;
    uint blockCount[2][MAX_NETWOP_COUNT];
    register uint cur_stream;
-   ulong  count1, count2;
+   uint32_t  count1, count2;
    uchar  ai_version[2];
    double avgPercentage1, avgPercentage2, variance1, variance2;
    uint   acqRepSum[2];
@@ -794,46 +1043,53 @@ bool EpgDbGetStat( CPDBC dbc, EPGDB_BLOCK_COUNT * pCount, time_t * acqMinTime, u
          // Cannot use the stream number from the block header because that's the stream
          // in which the block was received; However here we need the up-to-date stream
          // or the block counts will be inconsistent with the AI counts!
-         cur_stream = EpgDbGetStreamByBlockNo(dbc, pBlock);
-         if (pBlock->blk.pi.stop_time > now)
+         if (pBlock->blk.pi.block_no_in_ai == FALSE)
          {
-            pCount[cur_stream].allVersions += 1;
-            // note about the stream comparison:
-            // the borderline between stream 1 and 2 may change without a version change.
-            // since with a move from stream 2 to 1 more information may have been added
-            // we then consider the block out of date.
-            if ( (pBlock->version == ai_version[cur_stream]) &&
-                 (pBlock->stream == cur_stream) )
+            pCount[0].extra += 1;
+         }
+         else
+         {
+            cur_stream = EpgDbGetStreamByBlockNo(dbc, pBlock);
+            if (pBlock->blk.pi.stop_time > now)
             {
-               pCount[cur_stream].curVersion += 1;
+               pCount[cur_stream].allVersions += 1;
+               // note about the stream comparison:
+               // the borderline between stream 1 and 2 may change without a version change.
+               // since with a move from stream 2 to 1 more information may have been added
+               // we then consider the block out of date.
+               if ( (pBlock->version == ai_version[cur_stream]) &&
+                    (pBlock->stream == cur_stream) )
+               {
+                  pCount[cur_stream].curVersion += 1;
+               }
             }
-         }
-         else
-            pCount[cur_stream].expired += 1;
-
-         // note about "<=" in stream comparison: there's a bug in the TARA Nextview encoder,
-         // where blocks are assigned to stream 2 in AI, but transmitted in stream 1 anyways
-         // (e.g. when stream 1 is empty because there are no programmes in the time range
-         // that's usually covered by stream 1)
-         if ( (pBlock->acqTimestamp >= acqMinTime[cur_stream]) &&
-              (pBlock->version == ai_version[cur_stream]) &&
-              (pBlock->stream <= cur_stream) )
-         {
-            pCount[cur_stream].sinceAcq += 1;
-            blockCount[cur_stream][pBlock->blk.pi.netwop_no] += 1;
-         }
-
-         if (cur_stream == 1)
-         {  // stream 2
-            acqRepSum[cur_stream] += pBlock->acqRepCount;
-         }
-         else
-         {  // stream 1 needs special handling to split off rep counts from Now & Next
-            // PI in now & next sub-stream count exactly once; other PI may count more often
-            if (pBlock->acqRepCount >= maxNowRepCount)
-               acqRepSum[0] += 1;
             else
-               acqRepSum[0] += pBlock->acqRepCount;
+               pCount[cur_stream].expired += 1;
+
+            // note about "<=" in stream comparison: there's a bug in the TARA Nextview encoder,
+            // where blocks are assigned to stream 2 in AI, but transmitted in stream 1 anyways
+            // (e.g. when stream 1 is empty because there are no programmes in the time range
+            // that's usually covered by stream 1)
+            if ( (pBlock->acqTimestamp >= acqMinTime[cur_stream]) &&
+                 (pBlock->version == ai_version[cur_stream]) &&
+                 (pBlock->stream <= cur_stream) )
+            {
+               pCount[cur_stream].sinceAcq += 1;
+               blockCount[cur_stream][pBlock->blk.pi.netwop_no] += 1;
+            }
+
+            if (cur_stream == 1)
+            {  // stream 2
+               acqRepSum[cur_stream] += pBlock->acqRepCount;
+            }
+            else
+            {  // stream 1 needs special handling to split off rep counts from Now & Next
+               // PI in now & next sub-stream count exactly once; other PI may count more often
+               if (pBlock->acqRepCount >= maxNowRepCount)
+                  acqRepSum[0] += 1;
+               else
+                  acqRepSum[0] += pBlock->acqRepCount;
+            }
          }
 
          pBlock = pBlock->pNextBlock;
@@ -880,7 +1136,7 @@ bool EpgDbGetStat( CPDBC dbc, EPGDB_BLOCK_COUNT * pCount, time_t * acqMinTime, u
          if (count1 > 0)
          {
             pCount[0].ai += count1;
-            blockCount[0][netwop] = (uint)((ulong)1000L * blockCount[0][netwop] / count1);
+            blockCount[0][netwop] = (uint)(1000L * blockCount[0][netwop] / count1);
          }
          else
             blockCount[0][netwop] = 1000L;
@@ -889,7 +1145,7 @@ bool EpgDbGetStat( CPDBC dbc, EPGDB_BLOCK_COUNT * pCount, time_t * acqMinTime, u
          if (count2 > 0)
          {
             pCount[1].ai += count2;
-            blockCount[1][netwop] = (uint)((ulong)1000L * blockCount[1][netwop] / count2);
+            blockCount[1][netwop] = (uint)(1000L * blockCount[1][netwop] / count2);
          }
          else
             blockCount[1][netwop] = 1000L;
@@ -934,9 +1190,9 @@ bool EpgDbGetStat( CPDBC dbc, EPGDB_BLOCK_COUNT * pCount, time_t * acqMinTime, u
 // - modulo 0x10000
 // - if stop == start - 1  =>  distance == 0  (see ETS 300 707)
 //
-ulong EpgDbGetPiBlockCount( uint startNo, uint stopNo )
+uint EpgDbGetPiBlockCount( uint startNo, uint stopNo )
 {
-   ulong result;
+   uint result;
 
    if (stopNo >= startNo)
    {
