@@ -16,7 +16,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgstream.h,v 1.15 2002/05/10 18:01:34 tom Exp tom $
+ *  $Id: epgstream.h,v 1.16 2004/02/14 19:28:53 tom Exp tom $
  */
 
 #ifndef __EPGSTREAM_H
@@ -49,12 +49,28 @@ enum
   ,EPGDBACQ_TYPE_HI = 0x3F
 };
 
+typedef struct
+{
+   uint32_t  epgPagCount;       // number of EPG ttx pages received
+   uint32_t  epgPagMissing;     // number of missing pages
+   uint32_t  epgPkgCount;       // number of EPG ttx packets received
+   uint32_t  epgPkgMissing;     // number of missing packets
+   uint32_t  epgBiCount;        // number of BI blocks
+   uint32_t  epgBlkCount;       // number of blocks in stream (including BI)
+   uint32_t  epgBlkErr;         // number of dropped blocks (decoding error, not drop due to missing ttx pkg)
+   uint32_t  epgStrSum;         // number of string characters
+   uint32_t  epgParErr;         // number of parity errors
+   uint32_t  reserved_0;        // align to 8-byte boundary
+} EPG_STREAM_STATS;
+
 // ----------------------------------------------------------------------------
 // Declaration of service interface functions
 //
 void EpgStreamInit( EPGDB_QUEUE *pDbQueue, bool bWaitForBiAi, uint appId, uint pageNo );
 void EpgStreamClear( void );
 bool EpgStreamProcessPackets( void );
+bool EpgStreamCheckSlicerQuality( void );
+void EpgStreamGetStatistics( EPG_STREAM_STATS * pStats );
 
 
 #endif // __EPGSTREAM_H
