@@ -18,7 +18,7 @@
 #
 #  Author: Tom Zoerner
 #
-#  $Id: dlg_dump.tcl,v 1.3 2002/11/17 19:41:06 tom Exp tom $
+#  $Id: dlg_dump.tcl,v 1.6 2004/05/31 14:33:16 tom Exp tom $
 #
 set dumpdb_pi 1
 set dumpdb_xi 1
@@ -48,8 +48,13 @@ set dumphtml_popup 0
 set htmlcols_popup 0
 
 set dumpxml_filename {}
+set dumpxml_format 0
 set dumpxml_popup 0
 
+#=CONST= ::xmltv_dtd_5_gmt  0
+#=CONST= ::xmltv_dtd_5_ltz  1
+#=CONST= ::xmltv_dtd_6      2
+
 
 # notification by user-defined columns config dialog: columns addition or deletion
 proc DumpHtml_ColUpdate {} {
@@ -476,7 +481,7 @@ proc HtmlDump_Start {} {
 ##  Create dialog to export the database into TAB-separated text file
 ##
 proc PopupDumpXml {} {
-   global dumpxml_popup dumpxml_filename
+   global dumpxml_popup dumpxml_filename dumpxml_format
    global font_fixed fileImage
 
    if {$dumpxml_popup == 0} {
@@ -505,6 +510,13 @@ proc PopupDumpXml {} {
       pack   .dumpxml.all.name.dlgbut -side left -padx 5
       pack   .dumpxml.all.name -side top -pady 10
 
+      frame       .dumpxml.all.xml_fmt
+      radiobutton .dumpxml.all.xml_fmt.fmt0 -text "XML DTD 0.5 (UTC)" -variable dumpxml_format -value $::xmltv_dtd_5_gmt
+      radiobutton .dumpxml.all.xml_fmt.fmt1 -text "XML DTD 0.5 (local time zone)" -variable dumpxml_format -value $::xmltv_dtd_5_ltz
+      radiobutton .dumpxml.all.xml_fmt.fmt2 -text "XML DTD 0.6" -variable dumpxml_format -value $::xmltv_dtd_6
+      pack   .dumpxml.all.xml_fmt.fmt0 .dumpxml.all.xml_fmt.fmt1 .dumpxml.all.xml_fmt.fmt2 -side top -anchor w
+      pack   .dumpxml.all.xml_fmt -side top -pady 10 -anchor w
+
       frame  .dumpxml.all.cmd
       button .dumpxml.all.cmd.help -text "Help" -width 5 -command {PopupHelp $helpIndex(Control) "Export as XMLTV"}
       pack   .dumpxml.all.cmd.help -side left -padx 10
@@ -525,7 +537,7 @@ proc PopupDumpXml {} {
 
 # callback for Ok button
 proc DoDbXmlDump {} {
-   global dumpxml_popup dumpxml_filename
+   global dumpxml_popup dumpxml_filename dumpxml_format
 
    if [DlgDump_CheckOutputFile .dumpxml dumpxml_filename .xml 0 0] {
 
