@@ -16,18 +16,23 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: pdc_themes.h,v 1.3 2001/02/25 16:03:47 tom Exp tom $
+ *  $Id: pdc_themes.h,v 1.5 2002/07/20 16:28:14 tom Exp tom $
  */
 
 #ifndef __PDC_THEMES_H
 #define __PDC_THEMES_H
 
 
-extern const uchar * const pdc_themes[];
-extern const uchar * const pdc_series;
-extern const uchar * const pdc_undefined_theme;
-extern const uchar         pdc_categories[];
+#define PDC_THEME_SERIES  0x80
 
+// ----------------------------------------------------------------------------
+// Declaration of service interface functions
+//
+void PdcThemeSetLanguage( uchar lang );
+const uchar * PdcThemeGet( uchar theme );
+const uchar * PdcThemeGetByLang( uchar theme, uchar lang );
+const uchar * PdcThemeGetWithGeneral( uchar theme, const uchar ** pGeneralStr, bool withUndef );
+bool PdcThemeIsDefined( uchar theme );
 uchar PdcThemeGetCategory( uchar theme );
 
 #endif //__PDC_THEMES_H
